/* VIDEOCHATSCHOOL: VERSION 1
 * =====================================================
 * Last updated: July 05, 2007.
 * 
 * Assumes that SCHOOL is the server.
 * 
 * 
 * Equipment required
 * --------------------
 * Phidget Interface kit with the following: 
 * - Light sensor on the seat of the chair for detecting presence 
 * 
 */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace VideoChat
{
    public partial class VideoChatSchool : Form
    {
        // constants
        const int AWARENESS_INTERVAL = 3000;

        // variable declarations       
        bool present, twoWayTransmitting;
        Timer awarenessTimer;
        EasyImages.CameraClient wc;
        GroupLab.GTNetworking.SharedDictionary SD;        
        GroupLab.GTNetworking.Subscription imageSubscription;
        GroupLab.GTNetworking.Subscription pingSubscription;
        GroupLab.GTNetworking.Subscription textSubscription; 
        GroupLab.GTNetworking.Subscription isHereSubscription;
                       
        /* Thread handling: Need this block of code to make sure code runs from the main windows form, and not any other thread */
        public delegate void SetControlImage(PictureBox icontrol, Bitmap image); // delegate: pointer to a function
        public void RecievedImageToControl(PictureBox icontrol, Image image)
        {
            /* Control.InvokeRequired -- gets a value indicating whether the caller must call an invoke method when making method 
             * calls to the control because the caller is on a different thread than the one the control was created on. 
             * Control.BeginInvoke() -- executes a delegate asynchronously on the thread that the control's underlying handle was created on.
             */             
            if (icontrol.InvokeRequired) 
            {
                SetControlImage theDelegate = new SetControlImage(RecievedImageToControl);
                this.BeginInvoke(theDelegate, new object[] { icontrol, image });                
            }
            else
                icontrol.Image = image;
        }//RecievedImageToControl

        public VideoChatSchool()
        {
            Control.CheckForIllegalCrossThreadCalls = false;
            InitializeComponent();

            // Set up timers
            awarenessTimer = new Timer();
            awarenessTimer.Interval = AWARENESS_INTERVAL;
            
            // variable initializations
            wc = new EasyImages.CameraClient("DefaultCamera");            
            wc.ReceivedFrame += new EasyImages.CameraClient.ReceivedFrameEventHandler(wc_ReceivedFrame);
            wc.Start();
            present = false;            
            twoWayTransmitting = false;            
            this.cameraOnOffButton.Text = "Stop!";
            this.cameraOnOffButton.Enabled = false;
            
            // shared dictionary and subscription declarations
            SD = new GroupLab.GTNetworking.SharedDictionary();
            pingSubscription = new GroupLab.GTNetworking.Subscription();
            imageSubscription = new GroupLab.GTNetworking.Subscription();
            textSubscription = new GroupLab.GTNetworking.Subscription();
            isHereSubscription = new GroupLab.GTNetworking.Subscription();            
            
        }//Form1

        private void awarenessTimer_Tick(object sender, EventArgs e)
        {
            if (this.twoWayTransmitting) return; //if we are transmitting video two ways b/c both people are there, we don't want to use the awarenessTimer (send at normal frame rate)
            
            if (this.SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
            {       
               if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server) // Server (SCHOOL)
               { 
                    // Get normal frame to send over network using videoDisplay
                    this.SD["/ServerImage"] = new EasyImages.JPEG(wc.GetFrame(), 39);                    

                    // Display what is actually sent over the network
                    if (this.SD.Contains("/ServerImage"))
                        RecievedImageToControl(this.actualDisplay, (this.SD["/ServerImage"] as EasyImages.JPEG).GetImage()); // Display what is actually sent over network
                }
            }//if

        }//Form1

        private void wc_ReceivedFrame(object sender, EasyImages.CameraEventArgs e)
        {
            RecievedImageToControl(this.mirrorDisplay, e.Bitmap); //this.MirrorDisplay.Image = e.Bitmap;
            
        }//client_ReceivedFrame

        private void ConnectionButton_Click(object sender, EventArgs e)
        {
            if (this.connectionButton.Text == "Connect!")
            {
                string url = "tcp://localhost:hello";   // School (server) always runs off of localhost               
                if (DialogResult.OK == GroupLab.GTNetworking.UrlPrompt.Show(ref url))
                {
                    this.SD.Url = url;                    
                    this.SD.Opened += new EventHandler(SD_Opened);
                    this.SD.Closed +=  new EventHandler(SD_Closed);
                    this.SD.Open(); // start server connection
                    this.connectionButton.Text = "Disconnect!";
                    this.cameraOnOffButton.Enabled = true;

                    this.interfaceKit1.SensorChange += new GroupLab.SharedPhidgets.IndexSensorChangeEventHandler(this.interfaceKit1_SensorChange);
                    
                    awarenessTimer.Tick += new EventHandler(awarenessTimer_Tick);
                    awarenessTimer.Start();                    
                }
            }
            else
            {
                this.connectionButton.Text = "Connect!";
                this.SD.Close(); // close the connection with the shared dictionary
                this.cameraOnOffButton.Enabled = false;
            }
        }//ConnectionButton_Click

        private void SD_Opened(object sender, EventArgs e)
        {
            // imageSubscription
            this.imageSubscription.BeginInit();
            this.imageSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                this.imageSubscription.Pattern = "/ClientImage";
            this.imageSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(imageSubscription_Notified);
            this.imageSubscription.EndInit();

            // pingSubscription
            this.pingSubscription.BeginInit();
            this.pingSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                this.pingSubscription.Pattern = "/ClientPing";            
            this.pingSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(pingSubscription_Notified);
            this.pingSubscription.EndInit();

            // send the first frame
            //takeSnap();

            // textSubscription
            this.textSubscription.BeginInit();
            this.textSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                this.textSubscription.Pattern = "/ClientTextMessage";            
            this.textSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(textSubscription_Notified);
            this.textSubscription.EndInit();

            // isHereSubscription
            this.isHereSubscription.BeginInit();
            this.isHereSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)                
            {
                this.isHereSubscription.Pattern = "/ClientIsHere";
                this.SD["/ClientIsHere"] = false;
                this.SD["/ServerIsHere"] = false;
            }
            this.isHereSubscription.Notified +=new GroupLab.GTNetworking.SubscriptionEventHandler(isHereSubscription_Notified);
            this.isHereSubscription.EndInit();
                   
        }// SD_Opened
    

        private void SD_Closed(object sender, System.EventArgs args)
        {
        }//SD_Closed

        private void takeSnap()
        {
            if (this.SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
            {   
                if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                {                    
                    this.SD["/ServerImage"] = new EasyImages.JPEG(wc.GetFrame(), 39);
                }
            }//if
        }//takeSnap

        private void imageSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)//imageSubscription_Notified
        {
            EasyImages.JPEG j = e.Value as EasyImages.JPEG;
            if (j != null)
            {               
                RecievedImageToControl(this.videoDisplay, j.GetImage());  //this.VideoDisplay.Image = j.GetImage();                    
            }
           
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {
                // Display what is actually sent over network
                 if (this.SD.Contains("/ServerImage"))                    
                    RecievedImageToControl(this.actualDisplay, (this.SD["/ServerImage"] as EasyImages.JPEG).GetImage());

                // indicate that image has been recieved and can now get the next one (stop and wait)
                this.SD["/ServerPing"] = "ping!";
            }
        }//imageSubscription_Notified

        private void pingSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            if (twoWayTransmitting) takeSnap();
            
        }//pingSubscription_Notified

        private void textSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            this.textLabel.Text = e.Value.ToString();
        }//textSubscription_Notified
    
        private void checkPresence()
        {
            bool otherPresent = false; 

            if (SD.ConnectionType.Equals(GroupLab.GTNetworking.ConnectionType.Server))
                otherPresent = (bool)this.SD["/ClientIsHere"];  
                        
            // case 1: This is here, other isn't
            if (present && !otherPresent)
            {
                Console.WriteLine("State 1: this here, other isn't");                
                this.SD["/WaitingOnHome"] = true;            
                twoWayTransmitting = false;                
            }
            // case 2: This isn't here, other is  
            else if (!present && otherPresent)
            {
                Console.WriteLine("State 2: This not here, other is here");                
                twoWayTransmitting = false;
            }
            // case 3: both there
            else if (present && otherPresent)
            {
                Console.WriteLine("State 3: Both here");                
                twoWayTransmitting = true;               
                takeSnap();
            }
            // case 4: both gone
            else
            {
                Console.WriteLine("State 4: Both not here");                
                twoWayTransmitting = false;
            }
        }//checkPresences

        private void isHereSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            checkPresence();
        }//isHereSubscription_Notified

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // if user chooses to close the form manually, do so in an elegant fashion (stop camera and close dictionary)
            wc.Stop();
            if (SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
            {
                SD.Close();
            }
        }//Form1_FormClosing

        private void chatBox_KeyPress(object sender, KeyPressEventArgs e)
        {           
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {
                this.SD["/ServerTextMessage"] = chatTextBox.Text + e.KeyChar.ToString();
            }
            if (e.KeyChar == '\r')
            {
                chatTextBox.Text = "";
            }
        }//chatBox_KeyPress

        private void button1_Click(object sender, EventArgs e)
        {
            if (this.button1.Text.Equals("+"))
            {
                this.videoDisplay.Size = new System.Drawing.Size(320, 240);
                this.button1.Text = "-";

            }
            else if (this.button1.Text.Equals("-"))
            {
                this.videoDisplay.Size = new System.Drawing.Size(160, 120);
                this.button1.Text = "+";
            }//else
        }

        private void interfaceKit1_SensorChange(object sender, GroupLab.SharedPhidgets.IndexSensorChangeEventArgs e)
        {
            if (SD.Status != GroupLab.GTNetworking.ConnectionStatus.Opened) return;

            if (e.Index == 0)   // Light sensor on chair to detect presence
            {                
                if (e.Value < 100)
                {
                    present = true;                    
                    if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                    {
                        this.SD["/ServerIsHere"] = true;
                    }
                }
                else if (e.Value >= 100)
                {
                    present = false;
                    
                    if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                    {
                        this.SD["/ServerIsHere"] = false;
                        this.SD["/WaitingOnHome"] = false; 
                    }                 
                }

                checkPresence();

            }//if light sensor

        }//interfaceKit1_SensorChange

        private void cameraOnOffButton_Click(object sender, EventArgs e)
        {
            if (this.cameraOnOffButton.Text.Equals("Stop!"))
            {
                setCamera(0); // stop camera                
            }
            else
            {
                setCamera(1); // start camera              
            }

        }//cameraOnOffButton_Click

        private void setCamera(int action)
        {
            switch (action)
            {
                case 0:
                    wc.Stop();
                    awarenessTimer.Stop();
                    this.cameraOnOffButton.Text = "Start!";
                    this.twoWayTransmitting = false; // set to false because in ping_Notified, wont' be calling takeSnap to get the next frame (only send the stopped image)
                    Image i = Image.FromFile("stopped.jpg");

                    // show the stopped image in the actual display
                    RecievedImageToControl(this.actualDisplay, i);

                    // show stopped image in video display (depending on the connection type)
                    if (this.SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
                    {
                        if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                            this.SD["/ServerImage"] = new EasyImages.JPEG(i, 39);
                    }
                    break;

                case 1:
                    wc.Start();
                    awarenessTimer.Start();
                    this.awarenessTimer_Tick(null, null);
                    this.cameraOnOffButton.Text = "Stop!";
                    checkPresence();
                    break;

                default:
                    Console.WriteLine("setCamera default switch statement");
                    break;
            }
        }//setCamera

     
    }// partial class Form1
}// namespace VideoChat